@@ -79,6 +79,7 @@ enum {
 	DIRECTORY_ADDED,
 	DIRECTORY_REMOVED,
 	DIRECTORY_UPDATED,
+	CHILD_UPDATED,
 	LAST_SIGNAL
 };
 
@@ -315,11 +316,10 @@ tracker_indexing_tree_class_init (TrackerIndexingTreeClass *klass)
 	 * @indexing_tree: a #TrackerIndexingTree
 	 * @directory: a #GFile
 	 *
-	 * the ::directory-updated signal is emitted when @directory
-	 * that was previously added has had its indexing flags
-	 * updated due to another directory that is a parent of
-	 * @directory changing. This tends to happen uppon
-	 * tracker_indexing_tree_add() API calls.
+	 * The ::directory-updated signal is emitted on a root
+	 * when either its indexing flags change (e.g. due to consecutive
+	 * calls to tracker_indexing_tree_add()), or anytime an update is
+	 * requested through tracker_indexing_tree_notify_update().
 	 *
 	 * Since: 0.14.0
 	 **/
@@ -333,6 +333,30 @@ tracker_indexing_tree_class_init (TrackerIndexingTreeClass *klass)
 		              NULL,
 		              G_TYPE_NONE, 1, G_TYPE_FILE);
 
+	/**
+	 * TrackerIndexingTree::child-updated:
+	 * @indexing_tree: a #TrackerIndexingTree
+	 * @root: the root of this child
+	 * @child: the updated child
+	 *
+	 * The ::child-updated signal may be emitted to notify
+	 * about possible changes on children of a root.
+	 *
+	 * #TrackerIndexingTree does not emit those by itself,
+	 * those may be triggered through tracker_indexing_tree_notify_update().
+	 *
+	 * Since: 1.10
+	 **/
+	signals[CHILD_UPDATED] =
+		g_signal_new ("child-updated",
+		              G_OBJECT_CLASS_TYPE (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (TrackerIndexingTreeClass,
+		                               child_updated),
+		              NULL, NULL,
+		              NULL,
+		              G_TYPE_NONE, 2, G_TYPE_FILE, G_TYPE_FILE);
+
 	g_type_class_add_private (object_class,
 	                          sizeof (TrackerIndexingTreePrivate));
 }
@@ -594,6 +618,67 @@ tracker_indexing_tree_remove (TrackerIndexingTree *tree,
 	g_node_destroy (node);
 }
 
+/**
+ * tracker_indexing_tree_notify_update:
+ * @tree: a #TrackerIndexingTree
+ * @file: a #GFile
+ * @recursive: Whether contained indexing roots are affected by the update
+ *
+ * Signals either #TrackerIndexingTree::directory-updated or
+ * #TrackerIndexingTree::child-updated on the given file and
+ * returns #TRUE. If @file is not indexed according to the
+ * #TrackerIndexingTree, #FALSE is returned.
+ *
+ * If @recursive is #TRUE, #TrackerIndexingTree::directory-updated
+ * will be emitted on the indexing roots that are contained in @file.
+ *
+ * Returns: #TRUE if a signal is emitted.
+ *
+ * Since: 1.10
+ **/
+gboolean
+tracker_indexing_tree_notify_update (TrackerIndexingTree *tree,
+                                     GFile               *file,
+                                     gboolean             recursive)
+{
+	TrackerDirectoryFlags flags;
+	gboolean emitted = FALSE;
+	GFile *root;
+
+	g_return_val_if_fail (TRACKER_IS_INDEXING_TREE (tree), FALSE);
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	root = tracker_indexing_tree_get_root (tree, file, &flags);
+
+	if (tracker_indexing_tree_file_is_root (tree, file)) {
+		g_signal_emit (tree, signals[DIRECTORY_UPDATED], 0, root);
+		emitted = TRUE;
+	} else if (root &&
+	           ((flags & TRACKER_DIRECTORY_FLAG_RECURSE) ||
+	            g_file_has_parent (file, root))) {
+		g_signal_emit (tree, signals[CHILD_UPDATED], 0, root, file);
+		emitted = TRUE;
+	}
+
+	if (recursive) {
+		GList *roots, *l;
+
+		roots = tracker_indexing_tree_list_roots (tree);
+
+		for (l = roots; l; l = l->next) {
+			if (!g_file_has_prefix (l->data, file))
+				continue;
+
+			g_signal_emit (tree, signals[DIRECTORY_UPDATED], 0, l->data);
+			emitted = TRUE;
+		}
+
+		g_list_free (roots);
+	}
+
+	return emitted;
+}
+
 /**
  * tracker_indexing_tree_add_filter:
  * @tree: a #TrackerIndexingTree
@@ -1098,7 +1183,8 @@ prepend_config_root (GNode    *node,
 	GList **list = user_data;
 	NodeData *data = node->data;
 
-	*list = g_list_prepend (*list, data->file);
+	if (!data->shallow && !data->removing)
+		*list = g_list_prepend (*list, data->file);
 	return FALSE;
 }
 
